import string
import random


def generate_password():
    buffer = list(string.ascii_letters + string.digits + string.punctuation)
    length = random.randint(8, 15)
    password = "".join(random.sample(buffer, length))
    return password



#!/usr/bin/env python
import sys
import assets.password as password
from assets.management import create_database, query_db
from sqlite3 import Error
from getpass import getpass


def auth():
    p = getpass("Enter master password: ")
    if p == query_db("master")[2]:
        return True
    else:
        print("Invalid password")

# generate account
# account
# delete
# password

# todo: implement help system


if __name__ == "__main__":
    try:
        query_db("master")
    except Error:
        create_database()
        password.set_master_password()

    if auth():
        command = sys.argv[1:]
        if not command:
            raise Exception("pv.py help for available commands")

        switcher = {
            "generate": password.generate,
            "account": password.get,
            "accounts": password.get,
            "delete": password.delete,
            "save": password.save
        }

        event = switcher.get(command[0], "error")

        if not isinstance(event, str):
            event(command)
        else:
            raise Exception("Invalid argument")



import pyperclip
from getpass import getpass
from .management import (
    save_password,
    query_db,
    update_password,
    delete_account,
    get_all_accounts,
    delete_all_accounts
)
from .generate import generate_password


def confirmation(account, action):
    if action == "r":
        response = input("Are you sure you want to regenerate password for"
                         f" '{account}' (y/n): ")
    elif action == "d":
        response = input("Are you sure you want to delete password for"
                         f" '{account}' (y/n): ")
    elif action == "u":
        response = input("Are you sure you want to update password for"
                         f" '{account}' (y/n): ")
    elif action == "da":
        response = input("Are you sure you want to delete all saved passwords "
                         "(y/n): ")
    return response.lower() == "y"


def confirmation_message(account, action):
    if action == "g":
        print(f"Password generated for account: '{account}'"
              "\nCopied to clipboard!")

    if action == "r":
        print(f"Password regenerated for account: '{account}''"
              "\nCopied to clipboard!")


def generate(command):
    if len(command) > 1:
        account = command[1]
        password = generate_password()
        if not query_db(account):
            save_password(account, password)
            pyperclip.copy(password)
            confirmation_message(account, "g")
        else:
            if confirmation(account, "r"):
                update_password(account, password)
                pyperclip.copy(password)
                confirmation_message(account, "r")
    else:
        pyperclip.copy(generate_password())
        print("Random password copied to clipboard!")


def get(command):
    if len(command) > 1:
        account = command[1]
        password = query_db(account)[2]
        pyperclip.copy(password)
        print(f"Password copied to clipboard for '{account}'!")

    elif len(command) == 1:
        print("Accounts with passwords saved are:\n" +
              "\n".join(get_all_accounts()))


def delete(command):
    if len(command) > 1:
        account = command[1]
        if confirmation(account, "d"):
            delete_account(account)
            print(f"'{account}' deleted.")

    elif len(command) == 1:
        if confirmation(account, "da"):
            delete_all_accounts()
            print("All accounts deleted.")


def save(command):
    account = command[1]
    password = getpass(f"Enter password for '{account}': ")
    if not query_db(account):
        save_password(account, password)
        pyperclip.copy(password)
        print(f"Password saved for '{account}'!")
    else:
        if confirmation(account, "u"):
            update_password(account, password)
            pyperclip.copy(password)
            print(f"Password updated for '{account}'!")


def set_master_password():
    password = False
    again = True
    while again != password:
        password = getpass("Enter new master password: ")
        again = getpass("Enter new master password again: ")
    update_password("master", password)
    print("Master password set.")



import sqlite3


conn = sqlite3.connect("./passwords.db")


def create_database():
    """create database upon initialization"""
    with conn:
        c = conn.cursor()
        cmd = """CREATE TABLE IF NOT EXISTS passwords (
                    id integer PRIMARY KEY,
                    account text NOT NULL,
                    password text NOT NULL
                );
            """
        c.execute(cmd)
        save_password("master", "hash")


def query_db(account):
    with conn:
        c = conn.cursor()
        c.execute("SELECT * FROM passwords WHERE account=?", (account,))
        return c.fetchone()


def get_all_accounts():
    with conn:
        c = conn.cursor()
        c.execute("SELECT * FROM passwords")
        return [item[1] for item in c.fetchall()][1:]


def save_password(account, password):
    """save generated password to account in db"""
    with conn:
        c = conn.cursor()
        cmd = """INSERT INTO passwords(account, password)
                VALUES (?, ?)"""
        c.execute(cmd, (account, password))
    return c.lastrowid


def update_password(account, new_password):
    """updates an account with a new password"""
    with conn:
        c = conn.cursor()
        _id = query_db(account)[0]
        cmd = """UPDATE passwords
                 SET password = ?
                 WHERE id = ?"""
        c.execute(cmd, (new_password, _id))


def delete_account(account):
    """deletes entries for an account"""
    with conn:
        c = conn.cursor()
        _id = query_db(account)[0]
        cmd = "DELETE FROM passwords WHERE id = ?"
        if account == "all":
            cmd = "DELETE FROM passwords"
            c.execute(cmd)
            return
        c.execute(cmd, (_id,))


def delete_all_accounts():
    with conn:
        c = conn.cursor()
        c.execute("DELETE FROM passwords")



import setuptools

with open('requirements.txt') as f:
    requirements = f.readlines()

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pvault",
    version="1.0.0a",
    author="oluwafenyi",
    author_email="o.enyioma@gmail.com",
    description="A password manager package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/oluwafenyi/password-vault",
    license="MIT",
    package_name=["assets"],
    scripts=["pv.py", "assets/generate.py", "assets/management.py",
             "assets/password.py"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=requirements
)
